@@ -96,7 +96,7 @@ mod profile_view;
 
 pub use data::*;
 pub use merge::*;
-pub use profile_view::{FrameView, GlobalFrameView};
+pub use profile_view::{select_slowest, FrameView, GlobalFrameView};
 
 use std::collections::BTreeMap;
 use std::sync::atomic::{AtomicBool, Ordering};
@@ -123,10 +123,7 @@ pub type NanoSecond = i64;
 
 /// Stream of profiling events from one thread.
 #[derive(Clone, Default)]
-#[cfg_attr(
-    feature = "serialization",
-    derive(serde::Deserialize, serde::Serialize)
-)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct Stream(Vec<u8>);
 
 impl Stream {
@@ -194,10 +191,7 @@ pub struct Scope<'s> {
 
 /// Used to identify one source of profiling data.
 #[derive(Clone, Debug, Hash, Eq, PartialEq, Ord, PartialOrd)]
-#[cfg_attr(
-    feature = "serialization",
-    derive(serde::Deserialize, serde::Serialize)
-)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct ThreadInfo {
     /// Useful for ordering threads.
     pub start_time_ns: Option<NanoSecond>,
@@ -211,10 +205,7 @@ pub type FrameIndex = u64;
 
 /// A `Stream` plus some info about it.
 #[derive(Clone)]
-#[cfg_attr(
-    feature = "serialization",
-    derive(serde::Deserialize, serde::Serialize)
-)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct StreamInfo {
     /// The raw profile data.
     pub stream: Stream,
@@ -283,16 +274,18 @@ impl StreamInfo {
 
 /// One frame worth of profile data, collected from many sources.
 #[derive(Clone)]
-#[cfg_attr(
-    feature = "serialization",
-    derive(serde::Deserialize, serde::Serialize)
-)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct FrameData {
     pub frame_index: FrameIndex,
     pub thread_streams: BTreeMap<ThreadInfo, Arc<StreamInfo>>,
     pub range_ns: (NanoSecond, NanoSecond),
+    /// The uncompressed size
     pub num_bytes: usize,
     pub num_scopes: usize,
+
+    /// Size of the compressed data (in bytes), if this was decompressed.
+    #[cfg_attr(feature = "serde", serde(default, skip))]
+    pub compressed_size: Option<usize>,
 }
 
 impl FrameData {
@@ -324,6 +317,7 @@ impl FrameData {
                 range_ns: (min_ns, max_ns),
                 num_bytes,
                 num_scopes,
+                compressed_size: None,
             })
         } else {
             Err(Error::Empty)
@@ -380,32 +374,34 @@ impl FrameData {
             if &header == b"PFD0" {
                 let mut compressed_length = [0_u8; 4];
                 read.read_exact(&mut compressed_length)?;
-                let mut compressed = vec![0_u8; u32::from_le_bytes(compressed_length) as usize];
+                let compressed_length = u32::from_le_bytes(compressed_length) as usize;
+                let mut compressed = vec![0_u8; compressed_length];
                 read.read_exact(&mut compressed)?;
 
                 let serialized =
                     lz4_flex::decompress_size_prepended(&compressed).context("lz4 decompress")?;
 
                 use bincode::Options as _;
-                Ok(Some(
-                    bincode::options()
-                        .deserialize(&serialized)
-                        .context("bincode deserialize")?,
-                ))
+                let mut frame: Self = bincode::options()
+                    .deserialize(&serialized)
+                    .context("bincode deserialize")?;
+                frame.compressed_size = Some(compressed_length);
+                Ok(Some(frame))
             } else if &header == b"PFD1" {
                 let mut compressed_length = [0_u8; 4];
                 read.read_exact(&mut compressed_length)?;
-                let mut compressed = vec![0_u8; u32::from_le_bytes(compressed_length) as usize];
+                let compressed_length = u32::from_le_bytes(compressed_length) as usize;
+                let mut compressed = vec![0_u8; compressed_length];
                 read.read_exact(&mut compressed)?;
 
                 let serialized = zstd::decode_all(&compressed[..]).context("zstd decompress")?;
 
                 use bincode::Options as _;
-                Ok(Some(
-                    bincode::options()
-                        .deserialize(&serialized)
-                        .context("bincode deserialize")?,
-                ))
+                let mut frame: Self = bincode::options()
+                    .deserialize(&serialized)
+                    .context("bincode deserialize")?;
+                frame.compressed_size = Some(compressed_length);
+                Ok(Some(frame))
             } else {
                 anyhow::bail!("Failed to decode: this data is newer than this reader. Please update your puffin version!");
             }
@@ -595,7 +591,8 @@ impl GlobalProfiler {
             .append(stream_info);
     }
 
-    /// Call this function with each new finished frame.
+    /// Tells [`GlobalProfiler`] to call this function with each new finished frame.
+    ///
     /// The returned [`FrameSinkId`] can be used to remove the sink with [`Self::remove_sink`].
     pub fn add_sink(&mut self, sink: FrameSink) -> FrameSinkId {
         let id = self.next_sink_id;
@@ -611,15 +608,25 @@ impl GlobalProfiler {
 
 // ----------------------------------------------------------------------------
 
-/// Returns monotonically increasing nanosecond count.
-/// It is undefined when `now_ns()=0` is.
+/// Returns a high-precision, monotonically increasing nanosecond count since unix epoch.
 #[inline]
 pub fn now_ns() -> NanoSecond {
     // This can maybe be optimized
     use once_cell::sync::Lazy;
     use std::time::Instant;
-    static START_TIME: Lazy<Instant> = Lazy::new(Instant::now);
-    START_TIME.elapsed().as_nanos() as NanoSecond
+
+    fn epoch_offset_and_start() -> (NanoSecond, Instant) {
+        if let Ok(duration_since_epoch) = std::time::UNIX_EPOCH.elapsed() {
+            let nanos_since_epoch = duration_since_epoch.as_nanos() as NanoSecond;
+            (nanos_since_epoch, Instant::now())
+        } else {
+            // system time is set before 1970. this should be quite rare.
+            (0, Instant::now())
+        }
+    }
+
+    static START_TIME: Lazy<(NanoSecond, Instant)> = Lazy::new(epoch_offset_and_start);
+    START_TIME.0 + START_TIME.1.elapsed().as_nanos() as NanoSecond
 }
 
 // ----------------------------------------------------------------------------
@@ -732,6 +739,7 @@ fn test_short_file_name() {
     assert_eq!(short_file_name(r"C:\\windows\is\weird\src.rs"), "src.rs");
 }
 
+#[allow(clippy::doc_markdown)] // clippy wants to put "MacBook" in ticks 🙄
 /// Automatically name the profiling scope based on function name.
 ///
 /// Names should be descriptive, ASCII and without spaces.
@@ -759,7 +767,7 @@ fn test_short_file_name() {
 /// }
 /// ```
 ///
-/// Overhead: around 210 ns on 2020 MacBook Pro
+/// Overhead: around 210 ns on 2020 Intel MacBook Pro.
 #[macro_export]
 macro_rules! profile_function {
     () => {
@@ -786,6 +794,7 @@ macro_rules! profile_function_data {
     };
 }
 
+#[allow(clippy::doc_markdown)] // clippy wants to put "MacBook" in ticks 🙄
 /// Profile the current scope with the given name (unique in the parent scope).
 ///
 /// Names should be descriptive, ASCII and without spaces.
@@ -795,7 +804,7 @@ macro_rules! profile_function_data {
 /// An optional second argument can be a string (e.g. a mesh name) to help diagnose what was slow.
 /// Example: `profile_scope!("load_mesh", mesh_name);`
 ///
-/// Overhead: around 140 ns on 2020 MacBook Pro
+/// Overhead: around 140 ns on 2020 Intel MacBook Pro.
 #[macro_export]
 macro_rules! profile_scope {
     ($id:expr) => {
